/*   contrib/m_ojoin.c
 *   Copyright (C) 2002 Hybrid Development Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_ojoin.c,v 1.6 2004/04/24 14:40:05 stacy Exp $
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "channel.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "whowas.h"
#include "irc_string.h"
#include "hash.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "channel_mode.h"

static void mo_ojoin(struct Client *client_p, struct Client *source_p,
                         int parc, char *parv[]);

struct Message ojoin_msgtab = {
	"OJOIN", 0, 0, 2, 0, MFLG_SLOW, 0,
	{m_unregistered, m_not_oper, m_ignore, mo_ojoin}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
	mod_add_cmd(&ojoin_msgtab);
}

void
_moddeinit(void)
{
	mod_del_cmd(&ojoin_msgtab);
}

const char *_version = "$Revision: 1.6 $";
#endif

/*
** mo_ojoin
**      parv[0] = sender prefix
**      parv[1] = channel
*/
static void mo_ojoin(struct Client *client_p, struct Client *source_p,
                        int parc, char *parv[])
{
	struct Channel *chptr;
	int move_me = 0;

	/* opers only */
	if (!IsAnOper(source_p))
	{
		sendto_one(source_p, ":%s NOTICE %s :You have no O flag", me.name, parv[0]);
		return;
	}

	chptr = hash_find_channel(parv[1]);

	if(chptr == NULL)
	{
		sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL),
			   me.name, parv[0], parv[1]);
		return;
	}

	if(IsMember(source_p, chptr))
	{
		sendto_one(source_p, ":%s NOTICE %s :You are already on %s",
			   me.name, source_p->name, parv[1]);
		return;
	}

        if (is_banned(chptr, source_p) == CHFL_BAN)
	{
		sendto_one(source_p, form_str(ERR_BANNEDFROMCHAN),
			   me.name, parv[0], parv[1]);
                return;
	}

        if ((chptr->mode.mode & MODE_ADMINONLY) && (!IsOperAdmin(source_p)))
        {
                 sendto_one(source_p, form_str(ERR_ISADMINONLY),
                           me.name, parv[0], parv[1]);
                return;
        }

	if ((chptr->mode.mode & MODE_OPERONLY) && (!IsOper(source_p)))
	{
		sendto_one(source_p, form_str(ERR_ISOPERONLY),
			  me.name, parv[0], parv[1]);
		return;
	}

	if (move_me == 1)
		parv[1]--;

	sendto_wallops_flags(UMODE_OPERWALL, &me, 
			     "OPERWALL :OJOIN called for %s by %s!%s@%s",
			     parv[1], source_p->name, source_p->username, source_p->host);
	ilog(L_NOTICE, "OJOIN called for %s by %s!%s@%s",
	     parv[1], source_p->name, source_p->username, source_p->host);

	if(*chptr->chname != '&')
		sendto_server(NULL, NULL, NOCAPS, NOCAPS, 
			      ":%s OPERWALL :OJOIN called for %s by %s!%s@%s",
			      me.name, parv[1], source_p->name, source_p->username,
			      source_p->host);

	add_user_to_channel(chptr, source_p, CHFL_PEON);

	if(*chptr->chname != '&')
		sendto_server(client_p, chptr, NOCAPS, NOCAPS,  
			      ":%s SJOIN %lu %s + :%s",
			      me.name, chptr->channelts,
			      chptr->chname, source_p->name);

	sendto_channel_local(ALL_MEMBERS, chptr, ":%s!%s@%s JOIN %s",
			     source_p->name, source_p->username,
			     source_p->host, chptr->chname);

	/* send the topic... */
	if (chptr->topic != NULL)
	{
		sendto_one(source_p, form_str(RPL_TOPIC), me.name,
				source_p->name, chptr->chname, chptr->topic);
		sendto_one(source_p, form_str(RPL_TOPICWHOTIME), me.name,
				source_p->name, chptr->chname, chptr->topic_info,
				chptr->topic_time);
	}

	source_p->localClient->last_join_time = CurrentTime;
	channel_member_names(chptr, source_p, 1);
}

